//----------------------------------------------------------------------------------------------
// BSP : Seeed nRF52 Borads 1.1.1
// Board : Seeed nRF52 Borads / Seeed XIAO nRF52840 Sense 
//----------------------------------------------------------------------------------------------
// 2023/10/19
// 3.6V BAT pad (1S4 Vf=47mV)

#include <bluefruit.h>          // /Arduino15/packages/Seeeduino/hardware/nrf52/1.1.1/libraries/Bluefruit52Lib/src
#include <Adafruit_SPIFlash.h>  // Need to be deleted /Documents/Arduino/libraries/SdFat

#define HICHG         22        // Charge current　Hi:50mA Lo:100mA       D22 (13u)
#define CHG           23        // Charge LED Lo:ON                       D23 (17u)
#define VBAT_ENABLE   14        // Battery voltage read enable  Lo:enable D14 (14u)   
#define VBAT_READ     32        // Battery voltage read pin               D32 (31u)
// use D5 and D0 instead of P0.14 and P0.31
//#define VBAT_ENABLE   D5        // Battery voltage read enable  Lo:enable   
//#define VBAT_READ     D0        // Battery voltage read pin

// Built from the P25Q16H datasheet.
// https://gitlab.com/arduino5184213/seeed_xiao_nrf52840/flash_speedtest/-/tree/master
SPIFlash_Device_t const P25Q16H {
  .total_size = (1UL << 21),    // 2MiB
  .start_up_time_us = 10000,    // Don't know where to find that value  
  .manufacturer_id = 0x85,
  .memory_type = 0x60,
  .capacity = 0x15,
  .max_clock_speed_mhz = 55,
  .quad_enable_bit_mask = 0x02, // Datasheet p. 27
  .has_sector_protection = 1,   // Datasheet p. 27
  .supports_fast_read = 1,      // Datasheet p. 29
  .supports_qspi = 1,           // Obviously
  .supports_qspi_writes = 1,    // Datasheet p. 41
  .write_status_register_split = 1, // Datasheet p. 28
  .single_status_byte = 0,      // 2 bytes
  .is_fram = 0,                 // Flash Memory
};
Adafruit_FlashTransport_QSPI flashTransport;
Adafruit_SPIFlash flash(&flashTransport);

void setup()
{
  Serial.begin(115200);
  while(!Serial);
   
  // Enable DC-DC converter Mode
  NRF_POWER->DCDCEN = 1;            // Enable DC/DC converter for REG1 stage

  // initialise ADC wireing_analog_nRF52.c:73
  analogReference(AR_DEFAULT);      // default 0.6V*6=3.6V 　wireing_analog_nRF52.c:73
  analogReadResolution(12);         // wireing_analog_nRF52.c:39
  delay(1000);                      // Light Sleep  
    
  // on board Flash enter to Deep Power-Down Mode
  flashTransport.begin();
  flashTransport.runCommand(0xB9);  // enter deep power-down mode
  delayMicroseconds(5);             // tDP=3uS
  flashTransport.end();
  delay(1000);                      // Light Sleep

  // Pin initialization
  pinMode(CHG, INPUT_PULLUP);       // sleep current increases significantly without pull-up  
  pinMode(HICHG, OUTPUT);           // no effect on sleep current
  digitalWrite(HICHG, LOW);

 // VBAT ENABLE HIGH or LOW 
  pinMode(VBAT_ENABLE, OUTPUT);
  digitalWrite(VBAT_ENABLE, HIGH);  // vattery voltage read disable
//  digitalWrite(VBAT_ENABLE, LOW);   // vattery voltage read enable  
  delay(1000);                      // Light Sleep

  // Deep Sleep Mode
  NRF_POWER->SYSTEMOFF = 1;

  // even after entering deep sleep mode, remain 1-2uA
  //    SGM2040 3.3V LOD's quiescent current  (typ. 1.0uA, max. 1.5uA) 
  //                                 EN bias  (typ. 20nA, max.500nA)
  //    P25Q16H Flash Deep Power Down current (typ. 0.6uA, max. 3.0uA)
  //    nRF52840 DeepSleep Current            (typ. 0.4uA)
}

void loop()
{  
  digitalWrite(VBAT_ENABLE, LOW);   // vattery voltage read enable
  delay(5000);                      // Light Sleep

  // battery voltage read and get average  
  float vbatt = 0;
  for(int i = 0; i < 256; i++) {
    vbatt = vbatt + (2.961 * 3.6 * analogRead(VBAT_READ) / 4096);
  }
  Serial.println(vbatt / 256, 2);      
  delay(5000);                      // Light Sleep
  
  digitalWrite(VBAT_ENABLE, HIGH);  // vattery voltage read disable  
  delay(10000);                     // Light Sleep
}
